/*
"The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in
compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS"
basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
License for the specific language governing rights and limitations
under the License.

The Original Code is the content of this file.

The Initial Developer of the Original Code is Dr. Robert Kofler.
Carrer del Baluard; 73 21a
08003 Barcelona
All Rights Reserved.

Contributor(s): Dr. Robert Kofler.

Alternatively, the contents of this file may be used under the terms
of the _____ license (the  "[___] License"), in which case the
provisions of [______] License are applicable instead of those
above. If you wish to allow use of your version of this file only
under the terms of the [____] License and not to allow others to use
your version of this file under the MPL, indicate your decision by
deleting the provisions above and replace them with the notice and
other provisions required by the [___] License. If you do not delete
the provisions above, a recipient may use your version of this file
under either the MPL or the [___] License."
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SciRoKo
{
    public partial class ExtractFlank : Form
    {
        private System.Windows.Forms.Button bChoseOutputFile;
        private System.Windows.Forms.Button bChoseInputFile;
        private System.Windows.Forms.TextBox tbFlank3Prime;
        private System.Windows.Forms.Label l5PFlank;
        private System.Windows.Forms.Label l3PFlank;
        private System.Windows.Forms.TextBox tbFlank5Prime;
        private System.Windows.Forms.Label lExtractText;
        private System.Windows.Forms.TextBox tbInputFiles;
        private System.Windows.Forms.TextBox tbOutputFile;
        private System.Windows.Forms.GroupBox groupBox1;
        private System.Windows.Forms.Button bStartExtraction;
        private System.Windows.Forms.RadioButton rbFixedTotalLength;
        private System.Windows.Forms.RadioButton rbFlexibleFlank;

        public string LabelExtractText_Text
        {
            set
            {
                this.lExtractText.Text = value;
            }
        }
        public bool ButtonStartExtraction_Enabled
        {
            set
            {
                this.bStartExtraction.Enabled = value;
            }
        }
        public string TextBoxChoseOutputFile_Text
        {
            set
            {
                this.tbOutputFile.Text = value;
            }
        }
        public string TextBoxChoseInputFile_Text

        {
            set
            {
                this.tbInputFiles.Text = value;
            }
        }

        public string Label5PrimeFank_Text
        {
            set
            {
                this.l5PFlank.Text = value;
            }
        }
        public bool Label5PrimeFlank_Visble
        {
            set
            {
                this.l5PFlank.Enabled = value;
            }
        }
        public bool Label3PrimeFlank_Visible
        {
            set
            {
                this.l3PFlank.Enabled = value;
            }
        }
        public bool TextBox3PrimeFlank_Visible
        {
            set
            {
                this.tbFlank3Prime.Visible = value;
            }
        }
        public string TextBox3PrimeFlank_Text
        {
            set
            {
                this.tbFlank3Prime.Text = value;
            }
        }
        public string TextBox5PrimeFlank_Text
        {
            set
            {
                this.tbFlank5Prime.Text = value;
            }
        }
        public bool ButtonChoseOutputFile_Enabled
        {
            set
            {
                this.bChoseOutputFile.Enabled = value;
            }
        }


        private ExtractionControl control;

        public ExtractFlank(ExtractionControl control)
        {
            InitializeComponent();
            this.control = control;
        }

        private void ExtractFlank_Load(object sender, EventArgs e)
        {

        }

        private ExtractFlankingSettings GetExtractionSettings()
        {
            ExtractFlankingSettings settings = new ExtractFlankingSettings();
            if (rbFlexibleFlank.Checked == true)
            {
                settings.totalFlankMode = false;
                settings.flank5Prime = Convert.ToInt32(this.tbFlank5Prime.Text);
                settings.flank3Prime = Convert.ToInt32(this.tbFlank3Prime.Text);
            }
            else if (rbFixedTotalLength.Checked == true)
            {
                settings.totalFlankMode = true;
                settings.totalFlankLength = Convert.ToInt32(this.tbFlank5Prime.Text);
            }
            else throw new Exception("Invalid Radio Button status");
            return settings;
        }

        private void bChoseInputFile_Click(object sender, EventArgs e)
        {
            this.ofdChoseInput.InitialDirectory = control.StartupPath; //Load the startup path from the XmlDocument containing the settings
            this.ofdChoseInput.Filter = "Sequences in fasta (.fa .fasta .txt) |*.fa;*.txt;*.fasta";
            this.ofdChoseInput.Multiselect = true;
            if (this.ofdChoseInput.ShowDialog() == DialogResult.OK)
            {
                control.ButtonChoseInputFileClick(ofdChoseInput.FileNames);
               
            }
        }

        private void bChoseOutputFile_Click(object sender, EventArgs e)
        {
            this.sfdChoseOutput.InitialDirectory = control.StartupPath;
            this.sfdChoseOutput.Filter = "Sequences in fasta (.fasta)|*.fasta";

            if (this.sfdChoseOutput.ShowDialog() == DialogResult.OK)
            {
                control.ButtonChoseOutputFileClick(sfdChoseOutput.FileName);
            }

        }

        private void bStartExtraction_Click(object sender, EventArgs e)
        {
            control.ButtonStartExtractionClick(GetExtractionSettings());
        }

        private void rbFlexibleFlank_Click(object sender, EventArgs e)
        {
            control.FlankingModeChanged(this.rbFixedTotalLength.Checked==false);
        }

    }
}