/*
"The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in
compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS"
basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
License for the specific language governing rights and limitations
under the License.

The Original Code is the content of this file.

The Initial Developer of the Original Code is Dr. Robert Kofler.
Carrer del Baluard; 73 21a
08003 Barcelona
All Rights Reserved.

Contributor(s): Dr. Robert Kofler.

Alternatively, the contents of this file may be used under the terms
of the _____ license (the  "[___] License"), in which case the
provisions of [______] License are applicable instead of those
above. If you wish to allow use of your version of this file only
under the terms of the [____] License and not to allow others to use
your version of this file under the MPL, indicate your decision by
deleting the provisions above and replace them with the notice and
other provisions required by the [___] License. If you do not delete
the provisions above, a recipient may use your version of this file
under either the MPL or the [___] License."
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Collections;
using System.Xml;
using Bio.Repetitive.Microsat;
using SciRoKoSearchSupport;




namespace SciRoKo
{
    public interface ISciRoKoView
    {
        bool Button_DisplayStatisticsEnabled { get; set; }
        bool Button_ExportSSRsEnabled { get; set; }
        bool Button_LoadSequencesEnabled { get; set; }
        bool Button_LoadSSRsEnabled { get; set; }
        bool Button_SaveSearchResultsEnabled { get; set; }
        bool Button_StartSearchEnabled { get; set; }
        bool Button_ExtractFlankingSequenceEnabled { get;set;}
        bool CheckBox_ShowInfoEnabled { get; set; }
        bool CheckBox_ShowLengthInfoEnabled { get; set; }
        bool GroupBox_MotifStatisticsEnabled { get; set; }
        bool GroupBox_SearchResultSortingEnabled { get; set; }
        string Label_MismatchPenaltyText { set; }
        string Label_RequirementPerfectText { set; }
        bool RadioButton_DisplaySearchResultsEnabled { get; set; }
        bool RadioButton_DisplayStatisticsEnabled { get; set; }
        bool RadioButton_DisplayStatitsticsChecked { get;set;}
        string RichTextBox_AnalyzedText { set; }
        StringBuilder RichTextBox_Analyzed_StringBuilder { set; }

        string RichTextBox_FileStatusText { set; }
        bool TextBox_LowerBoundaryEnabled { get; set; }
        bool TextBox_MaxMismatchesAtOnceEnabled { get; set; }
        bool TextBox_MinimumRequiredScoreEnabled { get; set; }
        bool TextBox_MinRepeatsEnabled { get; set; }
        bool TextBox_MismatchPenaltyEnabled { get; set; }
        string TextBox_MismatchPenaltyText { set; }
        bool TextBox_MotifAssociationMaxDistanceEnabled { get; set; }
        bool TextBox_RequiredScoreEnabled { get; set; }
        bool TextBox_RequirementPerfectEnabled { get; set; }
        bool TextBox_SSRSeedLengthEnabled { get; set; }
        bool TextBox_SSRSeedRepeatsEnabled { get; set; }
        bool TextBox_UpperBoundaryEnabled { get; set; }
        bool CheckBox_LoadSearchIntoStatisticCheck { get;}
        string TextBox_RequirementPerfectText { set;}
        string LabelStartupPathText { set;}
        Bio.Repetitive.Microsat.SSRSearchMode SelectedSSRSearchMode { get;}
        SSRStatisticParameters StatisticParameters { get;}
        SSRSubsetBoundary SubsetBoundaries { get;}
    }

    public partial class SciRoKo : Form, ISciRoKoView
    {
        private SciRoKoControl control;
        private bool subsetChanged;

        private void SequenzeAnalyzerMain_Load(object sender, EventArgs e)
        {


        }



        public SciRoKo()
        {

            InitializeComponent();
            subsetChanged = false;
            this.control = new SciRoKoControl(this);
#if DEBUG
            string path = Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location);
            path = Path.Combine(path, "SciRoKoLogFile.log");
            StreamWriter sw = new StreamWriter(path, false);
            Debug.Listeners.Add(new TextWriterTraceListener(sw.BaseStream));
            Debug.WriteLine("DEBUG MODE, SciRoKo_3._");


            
#endif

        }

        /// <summary>
        /// Destructor ruft die Flush methode auf damit die TraceListener() den input noch schreiben.
        /// </summary>
        ~SciRoKo()
        {
            Debug.Flush();
        }

        //Important SciRoKo State dependend variables
        public bool Button_LoadSequencesEnabled
        {
            get
            {
                return this.bLoadSequences.Enabled;
            }
            set
            {
                this.bLoadSequences.Enabled = value;
            }
        }


        public bool Button_SaveSearchResultsEnabled
        {
            get
            {
                return this.bSaveSearchResults.Enabled;
            }
            set
            {
                this.bSaveSearchResults.Enabled = value;
            }
        }
        public bool Button_StartSearchEnabled
        {
            get
            {
                return this.bStartSearch.Enabled;
            }
            set
            {
                this.bStartSearch.Enabled = value;
            }
        }
       public bool Button_ExtractFlankingSequenceEnabled
       {
           get
           {
               return this.bExtractFlanking.Enabled;
           }
           set
           {
               this.bExtractFlanking.Enabled = value;
           }
       }

        public bool Button_LoadSSRsEnabled
        {
            get
            {
                return this.bLoadSSRs.Enabled;
            }
            set
            {
                this.bLoadSSRs.Enabled = value;
            }
        }
        public bool Button_ExportSSRsEnabled
        {
            get
            {
                return this.bExportSSRs.Enabled;
            }
            set
            {
                this.bExportSSRs.Enabled = value;
            }
        }
        public bool Button_DisplayStatisticsEnabled
        {
            get
            {
                return this.bDisplayStatistics.Enabled;
            }
            set
            {
                this.bDisplayStatistics.Enabled = value;
            }
        }
        public bool TextBox_MotifAssociationMaxDistanceEnabled
        {
            get
            {
                return this.tBMotifAssociationMaxDistance.Enabled;
            }
            set
            {
                this.tBMotifAssociationMaxDistance.Enabled = value;
            }
        }
        public bool GroupBox_MotifStatisticsEnabled
        {
            get
            {
                return this.gbMotifStatistics.Enabled;
            }
            set
            {
                this.gbMotifStatistics.Enabled = value;
            }
        }
        public bool CheckBox_ShowInfoEnabled
        {
            get
            {
                return this.cbShowInfo.Enabled;
            }
            set
            {
                this.cbShowInfo.Enabled = value;
            }
        }
        public bool CheckBox_ShowLengthInfoEnabled
        {
            get
            {
                return this.cbShowLengthInfo.Enabled;
            }
            set
            {
                this.cbShowLengthInfo.Enabled = value;
            }
        }
        public bool GroupBox_SearchResultSortingEnabled
        {
            get
            {
                return this.gbSearchResultSorting.Enabled;
            }
            set
            {
                this.gbSearchResultSorting.Enabled = value;
            }
        }
        public bool TextBox_MinimumRequiredScoreEnabled
        {
            get
            {
                return this.tbMinimumRequiredScore.Enabled;
            }
            set
            {
                this.tbMinimumRequiredScore.Enabled = value;
            }
        }
        public bool TextBox_LowerBoundaryEnabled
        {
            get
            {
                return this.tbLowerBoundary.Enabled;
            }
            set
            {
                this.tbLowerBoundary.Enabled = value;
            }
        }
        public bool TextBox_UpperBoundaryEnabled
        {
            get
            {
                return this.tbUpperBoundary.Enabled;
            }
            set
            {
                this.tbUpperBoundary.Enabled = value;
            }
        }
        public bool RadioButton_DisplayStatisticsEnabled
        {
            get
            {
                return this.rbDisplayStatistics.Enabled;
            }
            set
            {
                this.rbDisplayStatistics.Enabled = value;
            }
        }
        public bool RadioButton_DisplaySearchResultsEnabled
        {
            get
            {
                return this.rbDisplaySearchResults.Enabled;
            }
            set
            {
                this.rbDisplaySearchResults.Enabled = value;
            }
        }
        public bool RadioButton_DisplayStatitsticsChecked
        {
            get
            {
                return this.rbDisplayStatistics.Checked;
            }
            set
            {
                this.rbDisplayStatistics.Checked = value;
            }
        }

        //TextBoxes
        public string RichTextBox_AnalyzedText
        {
            set
            {
                this.rtbAnalyzed.Text = value;
            }
        }

        public StringBuilder RichTextBox_Analyzed_StringBuilder
        {
            set
            {
                this.rtbAnalyzed.Text = value.ToString();
            }
        }
        public string RichTextBox_FileStatusText
        {
            set
            {
                this.rtbFileStatus.Text = value;
            }
        }


        //SSR search mode dependend Variables
        public bool TextBox_RequirementPerfectEnabled
        {
            get
            {
                return this.tbRequirementPerfect.Enabled;
            }
            set
            {
                this.tbRequirementPerfect.Enabled = value;
            }
        }
        public bool TextBox_MinRepeatsEnabled
        {
            get
            {
                return this.tbMinRepeatsPerfect.Enabled;
            }
            set
            {
                this.tbMinRepeatsPerfect.Enabled = value;
            }
        }
        public bool TextBox_MismatchPenaltyEnabled
        {
            get
            {
                return this.tbMismatchPenalty.Enabled;
            }
            set
            {
                this.tbMismatchPenalty.Enabled = value;
            }
        }
        public bool TextBox_RequiredScoreEnabled
        {
            get
            {
                return this.tbRequiredScore.Enabled;
            }
            set
            {
                this.tbRequiredScore.Enabled = value;
            }
        }
        public bool TextBox_SSRSeedLengthEnabled
        {
            get
            {
                return this.tbSSRSeedLength.Enabled;
            }
            set
            {
                this.tbSSRSeedLength.Enabled = value;
            }
        }
        public bool TextBox_SSRSeedRepeatsEnabled
        {
            get
            {
                return this.tbssrSeedRepeats.Enabled;
            }
            set
            {
                this.tbssrSeedRepeats.Enabled = value;
            }
        }
        public bool TextBox_MaxMismatchesAtOnceEnabled
        {
            get
            {
                return this.tbMaxMismatchesAtOnce.Enabled;
            }
            set
            {
                this.tbMaxMismatchesAtOnce.Enabled = value;
            }
        }
        public string Label_MismatchPenaltyText
        {
            set
            {
                this.labelMismatchPenalty.Text = value;
            }
        }
        public string Label_RequirementPerfectText
        {
            set
            {
                this.labelRequiremPerfect.Text = value;
            }
        }
        public string TextBox_MismatchPenaltyText
        {
            set
            {
                this.tbMismatchPenalty.Text = value;
            }
        }
        public string TextBox_RequirementPerfectText
        {
            set
            {
                this.tbRequirementPerfect.Text = value;
            }
        }

        public bool CheckBox_LoadSearchIntoStatisticCheck
        {
            get
            {
                return cBSearchIntoStatistic.Checked;
            }
        }
        public SSRSubsetBoundary SubsetBoundaries
        {
            get
            {
                SSRSubsetBoundary boundaries = new SSRSubsetBoundary();
                boundaries.lowerBoundary = Convert.ToInt32(this.tbLowerBoundary.Text);
                boundaries.upperBoundary = Convert.ToInt32(this.tbUpperBoundary.Text);
                boundaries.minimumScore = Convert.ToInt32(this.tbMinimumRequiredScore.Text);
                return boundaries;
            }
        }
        public SSRStatisticParameters StatisticParameters
        {
            get
            {
                SSRStatisticParameters param = new SSRStatisticParameters();
                param.displayMotifAssociationStatistics = this.cbShowMotifAssociation.Checked;

                if (rbHomocouples.Checked == true) param.couplesSubtype = SSRStatisticParameters.CoupleSubtype.Homocouples;
                else if (rbHeterocouples.Checked == true) param.couplesSubtype = SSRStatisticParameters.CoupleSubtype.Heterocouples;
                else if (this.rbAll.Checked == true) param.couplesSubtype = SSRStatisticParameters.CoupleSubtype.All;
                else throw new Exception("Unconsidered state happen during determination of the SSR-statistic paramters");

                param.displayMotifLengthStatistics = this.cbShowLengthInfo.Checked;
                param.displayMotifStatistics = this.cbShowInfo.Checked;
                param.maxDistanceForAssociation = Convert.ToInt32(this.tBMotifAssociationMaxDistance.Text);
                param.minimumFrequencyInCount = Convert.ToInt32(this.tbMinCount.Text);
                param.motifAssoc_CS = this.cbType2.Checked;
                param.motifAssoc_FS = this.cbType1.Checked;
                param.motifAssoc_PSS = this.cbType4.Checked;
                param.motifAssoc_PSB = this.cbType3.Checked;
                param.motifStat_FS = this.cbFullyStandardized.Checked;
                param.motifStat_PS = this.cbPartiallyStandardized.Checked;

                if (this.rbFastaSSRStart.Checked) param.ssrSortMode = SSR.SSRSort.SSRSortMode.FirstSequenceNameSecondSSRStart;
                else if (this.rbMotifLengthATCG.Checked) param.ssrSortMode = SSR.SSRSort.SSRSortMode.FirstShortMotifsSecondATCG;
                else if (this.rbMismatchCount.Checked) param.ssrSortMode = SSR.SSRSort.SSRSortMode.MismatchCount;
                else if (this.rbScore.Checked) param.ssrSortMode = SSR.SSRSort.SSRSortMode.Score;
                else throw new Exception("Unconsidered state happen during determination of the SSR-statistic paramters");

                if (this.rbDetailed.Checked) param.statisticDetailLevel = Bio.Repetitive.Microsat.Statistics.SSRStatistics_DetailLevel.Detailed;
                else if (this.rbBasic.Checked) param.statisticDetailLevel = Bio.Repetitive.Microsat.Statistics.SSRStatistics_DetailLevel.Basic;
                else throw new Exception("Unconsidered state happen during determination of the SSR-statistic paramters");

                return param;

            }
        }
        public string LabelStartupPathText
        {
            set
            {
                this.lStartUpPath.Text = value;
            }
        }

        public Bio.Repetitive.Microsat.SSRSearchMode SelectedSSRSearchMode
        {
            get
            {
                SSRSearchMode searchMode;
                if (rbPerfectLength.Checked == true) searchMode = SSRSearchMode.PerfectLength;
                else if (rbPerfectRepeats.Checked == true) searchMode = SSRSearchMode.PerfectRepeats;
                else if (rbPerfectMISA.Checked == true) searchMode = SSRSearchMode.PerfectMISA;
                else if (rbMismatchedVariable.Checked == true) searchMode = SSRSearchMode.MisMatchVariablePenalty;
                else if (this.rbMismatchedFixed.Checked == true) searchMode = SSRSearchMode.MisMatchFixedPenalty;
                else throw new Exception("Invalid search mode");
                return searchMode;
            }
        }






        private void bReset_Click(object sender, EventArgs e)
        {
            control.ButtonResetClick();
        }


        private void bLoadSequences_Click(object sender, EventArgs e)
        {



            this.ofDLoadFiles.InitialDirectory = control.StartupPath; //Load the startup path from the XmlDocument containing the settings
            this.ofDLoadFiles.Filter = "Sequences in fasta (.fa .fasta .txt) |*.fa;*.txt;*.fasta";
            this.ofDLoadFiles.Multiselect = true;
            if (this.ofDLoadFiles.ShowDialog() == DialogResult.OK)
            {
                control.ButtonLoadSequencesClick(this.ofDLoadFiles.FileNames);

            }
        }


        private void bSaveSearchResults_Click(object sender, EventArgs e)
        {
            this.sfdSaveFiles.InitialDirectory = control.StartupPath;
            this.sfdSaveFiles.Filter = "SciRoKo-Format Recommended (.sciRo) |*.sciRo|Tab delimited (.td)|*.td";
            if (this.sfdSaveFiles.ShowDialog() == DialogResult.OK)
            {
                control.ButtonOutputFileClick(this.sfdSaveFiles.FileName);
            }
        }


        private void bLoadSSRs_Click(object sender, EventArgs e)
        {
            this.ofDLoadFiles.InitialDirectory = control.StartupPath; //Load the startup path from the XmlDocument containing the settings
            this.ofDLoadFiles.Filter = "SSRs in SciRoKo (.sciRo) |*.sciRo|Tab delimited (.td)|*.td|SSRs in Sputnik (.sput)|*.sput|SSRs in Modified Sputnik I (*.m1sput)|*.m1sput|SSRs in Modified Sputnik II (*.m2sput)|*.m2sput|SSR-Couples(*.ssrCou)|*.ssrCou";
            this.ofDLoadFiles.Multiselect = true;
            if (this.ofDLoadFiles.ShowDialog() == DialogResult.OK)
            {

                control.ButtonLoadSSRsClick(this.ofDLoadFiles.FileNames);

            }

        }


        private void bExportSSRs_Click(object sender, EventArgs e)
        {
            this.sfdSaveFiles.InitialDirectory = control.StartupPath; //Load the startup path from the XmlDocument containing the settings
            this.sfdSaveFiles.Filter = "SSRs in SciRoKo-Format (.sciRo) |*.sciRo|SSRs in Sputnik-Format (.sput) |*.sput|Tab delimited (.td)|*.td|SSRs in Modified Sputnik I (*.m1sput)|*.m1sput|SSRs in Modified Sputnik II (*.m2sput)|*.m2sput|SSR-Couples (*.ssrCou)|*.ssrCou";

            if (this.sfdSaveFiles.ShowDialog() == DialogResult.OK)
            {
                control.ButtonExportSSRsClick(sfdSaveFiles.FileName);



            }





        }


        private void bStartSearch_Click(object sender, EventArgs e)
        {
            SSRSearchSettings settings = new SSRSearchSettings();
            settings.searchMode = SelectedSSRSearchMode;
            if (settings.searchMode == SSRSearchMode.PerfectMISA)
            {
                settings.misaDefinition = this.tbRequirementPerfect.Text;
            }
            else if (settings.searchMode == SSRSearchMode.PerfectLength)
            {
                settings.minRep = Convert.ToInt32(this.tbMinRepeatsPerfect.Text);
                settings.minLength = Convert.ToInt32(this.tbRequirementPerfect.Text);

            }
            else if (settings.searchMode == SSRSearchMode.PerfectRepeats)
            {

                settings.minRep = Convert.ToInt32(this.tbRequirementPerfect.Text);
            }

            settings.requiredScore = Convert.ToInt32(this.tbRequiredScore.Text);
            settings.mismatchPenalty = Convert.ToInt32(this.tbMismatchPenalty.Text);
            settings.seedMinLength = Convert.ToInt32(this.tbSSRSeedLength.Text);
            settings.seedMinRep = Convert.ToInt32(this.tbssrSeedRepeats.Text);
            settings.mmao = Convert.ToInt32(this.tbMaxMismatchesAtOnce.Text);

            control.ButtonStartSearchClick(settings);


        }

        private void bDisplayStatistics_Click(object sender, EventArgs e)
        {

            control.ButtonDisplayStatisticClick();

        }

        /// <summary>
        /// SSR-search mode has been changed
        /// </summary>
        private void rbClick_Click(object sender, EventArgs e)
        {
            control.SearchModeChanged();
        }




        private void bSelectPath_Click(object sender, EventArgs e)
        {
            if (this.fbdSelectFolder.ShowDialog() == DialogResult.OK)
            {

                control.Button_PathSelectedClick(this.fbdSelectFolder.SelectedPath);

            }

        }







        private void SubsetChanged(object sender, EventArgs e)
        {
            this.subsetChanged = true;

        }


        /// <summary>
        /// Digesting large sequences
        /// </summary>
        private void bDigestSequences_Click(object sender, EventArgs e)
        {
            this.ofDLoadFiles.InitialDirectory = control.StartupPath; //Load the startup path from the XmlDocument containing the settings
            this.ofDLoadFiles.Filter = "Sequences in fasta (.fa .fasta .txt) |*.fa;*.txt;*.fasta";
            this.ofDLoadFiles.Multiselect = true;
            if (this.ofDLoadFiles.ShowDialog() == DialogResult.OK)
            {
                DigestionSettings settings = new DigestionSettings();
                settings.digestToSize = Convert.ToInt32(this.tbPretreatmentSize.Text);
                settings.overhead = Convert.ToInt32(this.tbPretreatmentOverhead.Text);
                control.Button_DigestSequencesClick(this.ofDLoadFiles.FileNames, settings);

            }
        }

        private void tabPMain_Enter(object sender, EventArgs e)
        {
            if (this.subsetChanged)
            {
                control.SubsetChanged();
                this.subsetChanged = false;
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            control.Button_ExtractFlankingSequencesClick();
        }





















    }



}
