/*
"The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in
compliance with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS"
basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
License for the specific language governing rights and limitations
under the License.

The Original Code is the content of this file.

The Initial Developer of the Original Code is Dr. Robert Kofler.
Carrer del Baluard; 73 21a
08003 Barcelona
All Rights Reserved.

Contributor(s): Dr. Robert Kofler.

Alternatively, the contents of this file may be used under the terms
of the _____ license (the  "[___] License"), in which case the
provisions of [______] License are applicable instead of those
above. If you wish to allow use of your version of this file only
under the terms of the [____] License and not to allow others to use
your version of this file under the MPL, indicate your decision by
deleting the provisions above and replace them with the notice and
other provisions required by the [___] License. If you do not delete
the provisions above, a recipient may use your version of this file
under either the MPL or the [___] License."
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using SciRoKoSearchSupport;

namespace SciRoKoCo
{
    class Program
    {
        static void Main(string[] args)
        {
            System.Threading.Thread.CurrentThread.CurrentCulture = new System.Globalization.CultureInfo("en-US");
#if DEBUG
            args = new string[] { "-i", @"G:\Temp\Sac\Sac.fasta","-mode","pl","-l","20", "-o", @"G:\Temp\Sac\Sacre2s.sciRo" };
#endif
            //



            SSRSearchSettings settings = new SSRSearchSettings();


            //Obligatory
            List<string> input = new List<string>();



            //Optional; set default
            string output = "results.sciRo";
            settings.minLength = 15;
            settings.minRep = 5;
            settings.misaDefinition = "14-7-5-4-4-4";
            settings.mismatchPenalty = 5;
            settings.mmao = 3;
            settings.requiredScore = 15;
            settings.searchMode = Bio.Repetitive.Microsat.SSRSearchMode.MisMatchFixedPenalty;
            settings.seedMinLength = 8;
            settings.seedMinRep = 3;


            //First determine the search mode and set some search mode specific default variables;
            bool error = false;
            for (int i = 0; i < args.Length; i++)
            {
                if (args[i] == "-mode")
                {
                    string temp = args[i + 1];
                    switch (temp)
                    {
                        case "pl":
                            settings.searchMode = Bio.Repetitive.Microsat.SSRSearchMode.PerfectLength;
                            settings.minRep = 3;
                            break;
                        case "pr": settings.searchMode = Bio.Repetitive.Microsat.SSRSearchMode.PerfectRepeats;
                            break;
                        case "misa": settings.searchMode = Bio.Repetitive.Microsat.SSRSearchMode.PerfectMISA;
                            break;
                        case "mmfp": settings.searchMode = Bio.Repetitive.Microsat.SSRSearchMode.MisMatchFixedPenalty;
                            break;
                        case "mmvp":
                            settings.searchMode = Bio.Repetitive.Microsat.SSRSearchMode.MisMatchVariablePenalty;
                            settings.mismatchPenalty = 1;
                            break;
                        default: error = true;
                            break;
                    }
                }
            }


            //Determine the parameters
            for (int i = 0; i < args.Length; i++)
            {
                if (args[i] == "-i") input.Add(args[i + 1]);
                if (args[i] == "-o") output = args[i + 1];

                //Perfect
                if (args[i] == "-l") settings.minLength = Convert.ToInt32(args[i + 1]);

                if (args[i] == "-r") settings.minRep = Convert.ToInt32(args[i + 1]);
                if (args[i] == "-m") settings.misaDefinition = args[i + 1];

                //Mismatched
                if (args[i] == "-s") settings.requiredScore = Convert.ToInt32(args[i + 1]);
                if (args[i] == "-p") settings.mismatchPenalty = Convert.ToInt32(args[i + 1]);
                if (args[i] == "-seedl") settings.seedMinLength = Convert.ToInt32(args[i + 1]);
                if (args[i] == "-seedr") settings.seedMinRep = Convert.ToInt32(args[i + 1]);
                if (args[i] == "-mmao") settings.mmao = Convert.ToInt32(args[i + 1]);
            }


            if (input.Count < 1) error = true;

            //Check for errors
            for (int i = 0; i < input.Count; i++)
            {
                FileInfo fi = new FileInfo(input[i]);
                if (!fi.Exists) error = true;
                input[i] = fi.FullName;
            }
            output = (new FileInfo(output)).FullName;

            if (settings.minRep < 2 || settings.mmao > 10 || settings.minLength < 8 || settings.seedMinRep < 2 || settings.seedMinLength < 8 || settings.requiredScore<8 || settings.mismatchPenalty<1) error = true;

            if (error)
            {

                Console.WriteLine("SciRoKoCo; The SciRoKo Console for Microsatellite Search ");
                Console.WriteLine("Parameters not sufficient or valid; Use default parameters if uncertain");
                Console.WriteLine("Example call: SciRoKoCo -i input1.fa -i input2.fa");
                Console.WriteLine("Example call: SciRoKoCo -i input.fasta -mode mmfp -s 15 -seedl 9 -seedr 4 -p 5 -o result.td");
                Console.WriteLine();
                Console.WriteLine("Obligatory parameters:");
                Console.WriteLine("-i           :input file(s); (multiple) fasta files; obligatory parameter");
                Console.WriteLine("             :several files may be specified");
                Console.WriteLine();
                Console.WriteLine("Optional parameters:");
                Console.WriteLine("-o           :output file; optional paramter; default: result.sciRo");
                Console.WriteLine("                Attention; the extension of the output file has to be either: '.sciRo' or '.td'");
                Console.WriteLine("                '.sciRo' is the recommendet output and '.td' is in tabdelimited for easy parsing");
                Console.WriteLine("-mode        :SSR-search mode; optional parameter; default: mmfp");
                Console.WriteLine("               possible modes are: pl, pr, misa, mmfp, mmvp");
                Console.WriteLine("               pl - perfect length: search for perfect microsatellites specify the minimum length");
                Console.WriteLine("               pr - perfect repeats: search for perfect microsatellites specify the minimum repeats");
                Console.WriteLine("               misa - misa mode: search for perfect microsatellites specify the minimum repeats for");
                Console.WriteLine("                 each microsatellite type individually, eg. 14-7-5-4-4-4 ");
                Console.WriteLine("               mmfp - mismatched fixed penalty: search for imperfect microsatellites using a");
                Console.WriteLine("                  minimum score and a fixed mismatch penalty (default mode)");
                Console.WriteLine("               mmvp - mismatched variable penalty: search for imperfect microsatellites using a");
                Console.WriteLine("                  minimum score and a variable mismatch penalty; penalty = motifLength x specifiedValue");
                Console.WriteLine();
                Console.WriteLine();
                Console.WriteLine("Parameters for pl-mode (perfect length):");
                Console.WriteLine("-l          :minimum length of a microsatellite in bp; optional parameter; default 15");
                Console.WriteLine("            :minimum = 8");
                Console.WriteLine("-r          :minimum repeats of a microsatellite; optional parameter; default 3");
                Console.WriteLine("            :minimum = 2");
                Console.WriteLine();
                Console.WriteLine("Parameters for pr-mode (perfect repeats):");
                Console.WriteLine("-r          :minimum repeats of a microsatellite; optional parameter; default 5");
                Console.WriteLine();
                Console.WriteLine("Parameters for misa-mode):");
                Console.WriteLine("-m          :minimum repeats for each microsatellite type; optional parameter; default 14-7-5-4-4-4");
                Console.WriteLine("             e.g: at least 14 repeats for a mononucleotide microsatellite");
                Console.WriteLine("                  at least 5 repeats for a trinucleotide microsatellite and so on");
                Console.WriteLine();
                Console.WriteLine("Parameters for mmfp-mode (mismatched fixed penalty):");
                Console.WriteLine("-s          :score; optional parameter; default 15");
                Console.WriteLine("            :minimum = 8");
                Console.WriteLine("-p          :mismatch penalty; optional parameter; default 5");
                Console.WriteLine("            :minimum = 1");
                Console.WriteLine("-seedl      :minimum length of a SSR-seed; optional parameter; default 8");
                Console.WriteLine("            :minimum = 8");
                Console.WriteLine("-seedr      :minimum repeats of a SSR-seed; optional parameter; default 3");
                Console.WriteLine("            :minimum = 2");
                Console.WriteLine("-mmao       :maximum mismatches at once; optional parameter; default 3");
                Console.WriteLine("              this value refers to the depth of the recursion, cpu time increases with this value ");
                Console.WriteLine("            :maximum = 10");
                Console.WriteLine();
                Console.WriteLine("Parameters for mmvp-mode (mismatched variable penalty):");
                Console.WriteLine("-s          :score; optional parameter; default 15");
                Console.WriteLine("            :minimum = 8");
                Console.WriteLine("-p          :variable mismatch penalty; optional parameter; default 1");
                Console.WriteLine("              the mismatch penalty is calculated = variablePenalty x SSR-motif length");
                Console.WriteLine("            :minimum = 1");
                Console.WriteLine("-seedl      :minimum length of a SSR-seed; optional parameter; default 8");
                Console.WriteLine("            :minimum = 8");
                Console.WriteLine("-seedr      :minimum repeats of a SSR-seed; optional parameter; default 3");
                Console.WriteLine("            :minimum = 2");
                Console.WriteLine("-mmao       :maximum mismatches at once; optional parameter; default 3");
                Console.WriteLine("            :maximum = 10");
                Console.WriteLine();
                Console.WriteLine();
                Console.WriteLine("Thank you for using SciRoKoCo");
                Console.WriteLine("Please report bugs to: robert@kofler.or.at");
                Console.WriteLine("You may display overall statistics for your microsatellite search using SciRoKo");
                return;
            }


                //SEARCH
                Model_SSRSearch model = new Model_SSRSearch();

                //Input files
                string[] tempFiles = new string[input.Count];
                for (int i = 0; i < input.Count; i++)
                {
                    tempFiles[i] = input[i];
                }
                model.SequenceFiles = tempFiles;


                //Output File
                model.SaveFile = output;
                model.StartSSRSearch(settings);

                Console.WriteLine("SSR search finished in {0} sec", model.SearchTime.TotalSeconds);
                Console.WriteLine("Thank you for using SciRoKoCo");


#if DEBUG
                Console.ReadLine();
#endif
            }

        }
    }

